#include "DatatypeConverterVisitor.h"

namespace armarx::aron::type::converter
{
    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::ObjectPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {TYPE_MEMBER_NAME}));
        dict->addElement(TYPE_MEMBER_NAME, type);

        for (const auto& [key, child] : el->getMemberTypes())
        {
            AronDatatypeConverterVisitor converter;
            aron::type::visit(converter, child);

            dict->addElement(key, converter.latest);
        }

        latest = dict;
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::DictPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {TYPE_MEMBER_NAME}));
        dict->addElement(TYPE_MEMBER_NAME, type);

        auto acceptedType = el->getAcceptedType();
        AronDatatypeConverterVisitor converter;
        aron::type::visit(converter, acceptedType);
        dict->addElement(ACCEPTED_TYPE_MEMBER_NAME, converter.latest);

        latest = dict;
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::ListPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {TYPE_MEMBER_NAME}));
        dict->addElement(TYPE_MEMBER_NAME, type);

        auto acceptedType = el->getAcceptedType();
        AronDatatypeConverterVisitor converter;
        aron::type::visit(converter, acceptedType);
        dict->addElement(ACCEPTED_TYPE_MEMBER_NAME, converter.latest);

        latest = dict;
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::PairPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {TYPE_MEMBER_NAME}));
        dict->addElement(TYPE_MEMBER_NAME, type);

        auto acceptedType1 = el->getFirstAcceptedType();
        AronDatatypeConverterVisitor converter1;
        aron::type::visit(converter1, acceptedType1);
        dict->addElement(FIRST_ACCEPTED_TYPE_MEMBER_NAME, converter1.latest);

        auto acceptedType2 = el->getSecondAcceptedType();
        AronDatatypeConverterVisitor converter2;
        aron::type::visit(converter2, acceptedType2);
        dict->addElement(SECOND_ACCEPTED_TYPE_MEMBER_NAME, converter2.latest);

        latest = dict;
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::TuplePtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {TYPE_MEMBER_NAME}));
        dict->addElement(TYPE_MEMBER_NAME, type);

        int i = 0;
        for (const auto& child : el->getAcceptedTypes())
        {
            AronDatatypeConverterVisitor converter;
            aron::type::visit(converter, child);

            dict->addElement(std::string(ACCEPTED_TYPE_MEMBER_NAME) + "_" + std::to_string(i++),
                             converter.latest);
        }

        latest = dict;
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::MatrixPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::NDArrayPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::QuaternionPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::PointCloudPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::ImagePtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::IntEnumPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::IntPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::LongPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::FloatPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::DoublePtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::BoolPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::StringPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void
    AronDatatypeConverterVisitor::visitAronVariant(const type::AnyObjectPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }
} // namespace armarx::aron::type::converter
